# from time import time
# from unittest import TestCase

# @classmethod
# def setUpClass(cls):
#     cls.startTime = time()


# @classmethod
# def tearDownClass(cls):
#     print("\n%s.%s: %.3f" % (cls.__module__, cls.__name__, time() - cls.startTime))


# TestCase.setUpClass = setUpClass
# TestCase.tearDownClass = tearDownClass
